#!/usr/bin/env ruby

$VERBOSE = nil

Signal.trap('INT' ){ puts ''; exit }

loop do
  list = `deluge-console info | sed 's/^\s*$/--/'`.split('--').map{|info|
    next if info.strip.size == 0

    info.strip.split("\n").inject({'Progress' => 0.0}){|h, l|
      k, v = l.match(/([^:]+): (.+)/).captures rescue %w{ None none }
      v = v.sub(/(.+) Peers.+Availability: (.+)/, '\1 / \2%')      if k == 'Seeds'
      v = v.sub(/.+\/([0-9\.]+ [^ ]+).+/        , '\1'      )      if k == 'Size'
      v = v.sub(/([0-9\.]+)%.+/                 , '\1'      ).to_f if k == 'Progress'
      v = v.sub(/Downl[^:]+: ([0-9\.]+ [^ ]+).+/, '\1'      )      if k == 'State'
      v = v.sub(/(Seeding).+/                   , '\1'      )      if k == 'State'
      h.merge k => v
    }
  }.compact.sort{|a,b| a['Progress'] <=> b['Progress'] }

  # https://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
  term_w = `tput cols`.to_i - 4 # ENV['COLUMNS'].to_i
  max_l  = term_w - 4 - 10 - 11 - 17 - 4

  speed = 0

  list.each do |t|
    # count UTF8 chars as double width chars
    name = ''
    t['Name'].each_char.inject(0) do |s, c|
      s += c.codepoints[0] > 255 ? 2 : 1
      break unless s <= max_l
      name += c
      s
    end

    info = [
      '%3d%%' % t['Progress'],
      t['Size' ].to_s.rjust(10),
      t['State'].to_s.rjust(11),
      t['Seeds'].to_s.rjust(17),
      name
    ].join('|')

    if    t['Seeds'] =~ /^0 \(0\)/; puts "! #{info}"
    elsif t['State'] =~ /Paused/  ; puts "P #{info}"
    elsif t['State'] =~ /Checking/; puts "C #{info}"
    elsif t['State'] =~ /Seeding/ ; puts "S #{info}"
    else                            puts "  #{info}"
      speed += t['State'].to_f        if t['State'] =~ /KiB/
      speed += t['State'].to_f * 1024 if t['State'] =~ /MiB/
    end
  end

  puts '-'*(term_w-2)
  puts "Downloading #{list.size} @ #{speed.round(2)} KiB/s"
  
  break unless ARGV.include?('-l')

  sleep 30
  puts "\e[H\e[J" # `clear`
end # loop